<?php
/*
Plugin Name: RePress plugin
Plugin URI: https://all4xs.net
Description: RePress plugin
Version: 0.1alpha6

Future features:

    - set max sizes for transfers
    - support hostnames as hashes
    - support HTTPS

*/

// Hackers -> go to proxy.php

// css file
wp_enqueue_style( 'repress_css', get_bloginfo('wpurl') . '/wp-content/plugins/repress/repress.css', 'false', '0.1', 'all' );

// add activation hooks
register_activation_hook(__FILE__, 'activate_repress');
register_deactivation_hook(__FILE__, 'deactivate_repress');

function activate_repress() {
	// activate the plugin. initialize the options with the defaults
	// and add some well known censored sites to the database

	add_option('repress_errormsg', '');
	add_option('repress_rewritebase', '/repress');
	add_option('repress_remoteservers', json_encode ( array  (
							'wikipedia.org',
							'wikimedia.org',
							'wikileaks.org',
							'piratebay.org',
							'thepiratebay.org',
							'newton.baycdn.net',
							'piratebay.cc'
						) )
		);
	add_option('repress_push_post', 'yes');
	add_option('repress_push_cookie', 'no');
	add_option('repress_secret', makeSecret());	// future work: use secret key to support hashes in URLs
	add_option('repress_version', plugin_get_version());

	// statistics
	add_option('repress_bandwidth_usage_bytes', 0);
	add_option('repress_bandwidth_usage_mb', 0);
}

function deactivate_repress() {
	// deactivate the plugin. remove options

        delete_option('repress_errormsg');
        delete_option('repress_rewritebase');
        delete_option('repress_remoteservers');
        delete_option('repress_push_post');
        delete_option('repress_push_cookie');
        delete_option('repress_secret');
        delete_option('repress_version');
        delete_option('repress_bandwidth_usage_bytes');
        delete_option('repress_bandwidth_usage_mb');

}

// provide a menu in the administration interface to change options
add_action('admin_menu', 'repress_modify_menu');

function repress_modify_menu() {
	add_options_page(	'Repress',
				'RePress',
				'manage_options',
				__FILE__,
				'repress_set_options'
			);
}

// called in administration backend for options screen
function repress_set_options() {

	if ((array_key_exists('submit', $_REQUEST) && $_REQUEST['submit']) ||
            (array_key_exists('uncensor', $_REQUEST) && $_REQUEST['uncensor']) ||
	    (array_key_exists('repressdelete', $_REQUEST) && $_REQUEST['repressdelete'])) {
		repress_update_options();
	}

	if (array_key_exists('save', $_REQUEST) && $_REQUEST['save']) {
		repress_update_options_save();
	}

	repress_draw_form();
}

// html form to set plugin options
function repress_draw_form() {
	if (get_option('repress_push_post') == 'yes') {
		$checkedPushPost = 'checked';
	} else {
		$checkedPushPost = '';
	}
	if (get_option('repress_push_cookie') == 'yes') {
		$checkedPushCookie = 'checked';
	} else {
		$checkedPushCookie = '';
	}

	?>
		<div class="wrap"><h2>RePress options</h2>
		<div id="rpside">
			<a href="https://all4xs.net"><img src="<?=repress_draw_logo_url();?>" style="float: right"></a>
			<div id="rpundertext">Version <?=plugin_get_version();?></div>
			Designed by <a href="https://all4xs.net">all4xs.net</a><br /><br />
			html parser by<br />
			<a href="http://code.google.com/p/phproxyimproved/">phproxyimproved</a>
			<p>
			<b>Statistics</b>
			</p>
			<p>
			<?=repress_draw_megabytes();?> mb proxied
			</p>
		</div>
		<?php
		if (get_option('repress_errormsg') !== '') {
			echo '<div id="rperror">' . get_option('repress_errormsg') . '</div>';
			repress_clean_error();
		}
		?>
		<br />
		RePress requires a unique permalink. This is the address under which the proxy will function.<br />
		This must be a single word. Leading slash is optional.<br />
		<p>
		<form method="post">
			<label for 'repress_rewritebase'>RePress permalink:
				<input type="text" name="repress_rewritebase" value="<?=get_option('repress_rewritebase');?>" />
			</label>
			&nbsp;&nbsp;<input type="submit" name="submit" value="Update" />
		</form>
		</p>
		</div>
		<div class="wrap"><h3>Uncensored domains</h3>
		The following domains are uncensored by RePress.<br />
		<p>
		<?=repress_draw_domaintable();?>
		</p>
		Free a domain from the clutches of tiranny (please type only the hostname, not HTTP).
		<form method="post">
			<label for 'repress_uncensordomain'>Add domainname:
				<input type="text" name="repress_uncensordomain" value="" />
			</label>
			&nbsp;&nbsp;<input type="submit" name="uncensor" value="Uncensor!" />
		</form>
		</div>
		<div class="wrap"><h3>Advanced options</h3>
		<form method="post">
			<label for 'repress_push_post'>Push POST variables:
				<input type="checkbox" name="repress_push_post" <?=$checkedPushPost;?>/>
			</label>
			<br />
			<label for 'repress_push_cookie'>Allow cookies:
				<input type="checkbox" name="repress_push_cookie" <?=$checkedPushCookie;?>/>
			</label>
			<br /><br />
			<input type="submit" name="save" value="Save" />
		</form>
		</div>
	<?
}

// html rendering of logout page
function repress_draw_logoutpage() {
	header('Content-Type: text/html',true);

	?>
	<html>
	<head>
	<title>Please log out of wordpress</title>
	</head>
	<body>
	<html>
	In order to protect your blog from cross-site scripting attacks, please log out of your Wordpress administration backend before accessing any of the proxied site urls.<br />
	<br />
	Please log out of Wordpress by clicking on this <a href="<?php echo wp_logout_url( $_SERVER['REQUEST_URI'] ); ?>" title="Logout">logout</a> link.<br />
	Afterwards you will be directly redirected to the proxied website.
	</body>
	</html>

	<?
}

// html rendering of the domain table
function repress_draw_domaintable() {
	$table = '<div id="rpdomaintable"><table>';

	$table .= '<tr>' . '<th width="80%">Domain</th>' . '<th>Remove</th>' . '<th>Visit</th>' . '</tr>';

	$domains = json_decode( get_option('repress_remoteservers'), TRUE );
	if (is_array($domains)) {
		foreach ($domains as $host) {
			$table .= '<tr>' . "<td>$host</td>" . "<td>" . repress_draw_deletehost($host) . "</td>" . "<td>" . repress_draw_site_link($host, 'Visit') . "</td></tr>";
		}
	}

	$table .= '</table></div>';
	return $table;
}

// link to logo image
function repress_draw_logo_url() {
	return get_bloginfo('wpurl') . '/wp-content/plugins/repress/img/logo.jpeg';
}

// make link to delete host from domain table
function repress_draw_deletehost($host) {
	return '<a href = "' . $_SERVER['REQUEST_URI'] . '&repressdelete=' . urlencode($host) . '">Delete</a>';
}

// display bandwidth
function repress_draw_megabytes() {
	return get_option('repress_bandwidth_usage_mb');
}

// html rendering of overview of uncensored sites
function repress_draw_overview_page() {
	header('Content-Type: text/html',true);
?>
	<html>
	<head>
	<title>RePress web proxy provider</title>
	</head>
	<body>
	<a href="https://all4xs.net" style="float:left"><img src="<?=repress_draw_logo_url();?>"></a>
	<h1 style="float:up">Uncensor the web using <a href="https://all4xs.net">RePress.</a></h1>
	<h3 style="margin-left:250px">Stop internet censorship now!</h3><br /><br /><br />
	The following sites are accessible through this webproxy:
	<br /><br />
<?php
	$domains = json_decode( get_option('repress_remoteservers'), TRUE );
	if (is_array($domains)) {
		sort($domains, SORT_STRING);
		foreach ($domains as $host) {
			echo repress_draw_site_link($host, $host);
			echo "<br />";
		}
	}
?>
	</body>
	</html>
<?php

}

// link to a uncensored site
function repress_draw_site_link($host, $comment) {
	// future work: support hashes in links
	return '<a href="' . get_option('repress_rewritebase') . '/' . $host . '/">' . $comment . '</a>';
}

// handle updating of options
function repress_update_options() {
	if (array_key_exists('repress_rewritebase', $_REQUEST) && $_REQUEST['repress_rewritebase']) {
		// validation

		if (!is_string($_REQUEST['repress_rewritebase'])) {
			repress_produce_error('The base URL is not a string!');
			return;
		}

		// remove beginning and trailing slashes
		$_REQUEST['repress_rewritebase'] = preg_replace("/^\/*/", "", $_REQUEST['repress_rewritebase']);
		$_REQUEST['repress_rewritebase'] = preg_replace("/\/*$/", "", $_REQUEST['repress_rewritebase']);

		if ($_REQUEST['repress_rewritebase'] == '') {
			repress_produce_error('Running RePress on the root of your website is not supported!');
			return;
		}

		// now validate: only alphanumerical characters
		if (preg_match("/[^a-zA-Z+d]/", $_REQUEST['repress_rewritebase'])) {
			repress_produce_error('The base URL may only contain alphanumerical characters.');
			return;
		}

		// add single leading slash
		$_REQUEST['repress_rewritebase'] = '/' . $_REQUEST['repress_rewritebase'];

		// set option	
		update_option('repress_rewritebase', $_REQUEST['repress_rewritebase']);
	}

	if (array_key_exists('repressdelete', $_REQUEST) && $_REQUEST['repressdelete']) {

		$hosts = json_decode( get_option('repress_remoteservers'), TRUE );
		$del = $_REQUEST['repressdelete'];
		$copy = array();
		foreach ($hosts as $host) {
			if ($host !== $del) {
				$copy[] = $host;
			}
		}
		$_SERVER['REQUEST_URI'] = preg_replace("#&repressdelete=.*$#", "", $_SERVER['REQUEST_URI']);
		update_option('repress_remoteservers', json_encode( $copy ));
	}

	if (array_key_exists('repress_uncensordomain', $_REQUEST) && $_REQUEST['repress_uncensordomain']) {

		// validation

		$hostname = $_REQUEST['repress_uncensordomain'];

		if (preg_match("/(?:(?:(?:(?:[a-zA-Z0-9][-a-zA-Z0-9]*)?[a-zA-Z0-9])[.])*(?:[a-zA-Z][-a-zA-Z0-9]*[a-zA-Z0-9]|[a-zA-Z])[.]?)/", $hostname)) {

			// make sure we do not have any duplicates, then add

			$hosts = json_decode( get_option('repress_remoteservers'), TRUE );

			if (in_array($hostname, $hosts)) {
				repress_produce_error('Duplicate host.');
				return;
			}

			$hosts[] = $hostname;
			$_SERVER['REQUEST_URI'] = preg_replace("#&repress_uncensordomain=.*$#", "", $_SERVER['REQUEST_URI']);
			update_option('repress_remoteservers', json_encode( $hosts ));
		
			
		} else {
			repress_produce_error('The host name appears to be invalid.');
			return;
		}

	}

}


// checkboxes save
function repress_update_options_save() {

	if (array_key_exists('repress_push_post', $_REQUEST) && $_REQUEST['repress_push_post']) {
		update_option('repress_push_post', 'yes');
	} else {
		update_option('repress_push_post', '');
	}

	if (array_key_exists('repress_push_cookie', $_REQUEST) && $_REQUEST['repress_push_cookie']) {
		update_option('repress_push_cookie', 'yes');
	} else {
		update_option('repress_push_cookie', '');
	}

}

// store error feedback message to be shown after page refresh
function repress_produce_error($str) {
	update_option('repress_errormsg', $str);
}

// clean error message
function repress_clean_error() {
	update_option('repress_errormsg', '');
}

// get current version of the plugin
function plugin_get_version() {

	if ( function_exists('get_plugin_data') ) {
		$plugin_data = get_plugin_data( __FILE__ );
		$plugin_version = $plugin_data['Version'];
		update_option('repress_version', $plugin_version );
		return $plugin_version;
	}

	$plugin_version = get_option('repress_version');
	if (is_string($plugin_version) && $plugin_version !== '') {
		return $plugin_version;
	}

	return 'unknown';
}

// try to protect against XSS attacks by logging out the Wordpress user
function repress_logout_wordpress_user() {
	if (is_user_logged_in() == TRUE) {
		repress_draw_logoutpage();
		exit;
	}
}

// did we surf to the root of the rewrite permalink itself? draw overview of uncensored sites

if (preg_replace("#/$#", "", $_SERVER['REQUEST_URI']) == get_option('repress_rewritebase')) {
	repress_draw_overview_page();
	exit;
}

add_action('init', 'repress_proxy');

function repress_proxy() {

	// require plugin proxy code. which runs directly.

	require_once('proxy.php');


}

/** Bandwidth accounting **/

function repress_register_bandwidth_usage($size) {

	// in wordpress plugin, keep this in database

	$usage_now_bytes = get_option('repress_bandwidth_usage_bytes');

	$usage_new_bytes = $usage_now_bytes + $size;
	
	if ($usage_new_bytes > 1073741824) {

		// another megabyte has been transferred

		$usage_new_bytes -= 1073741824;
		$usage_now_mb = get_option('repress_bandwidth_usage_mb');

		update_option('repress_bandwidth_usage_mb', $usage_now_mb + 1);
	}

	update_option('repress_bandwidth_usage_bytes', $usage_new_bytes);
}


/** Hash functions. If mrand library is not loaded, we will use hostnames in our urls. **/
/** (future work: this is not used yet.) **/

function makeSecret() {

	$secret = '';
	for ($i = 0; $i < 8; $i++) {
		$secret .= chr(mt_rand(0, 255));
	}

	return $secret;
}

function getHostFromhash($hash, $secret) {

        if (function_exists('mcrypt_ecb')) {

                $decrypted = mcrypt_ecb(MCRYPT_BLOWFISH, $secret, base64_decode( $hash ), MCRYPT_DECRYPT );

                return $decrypted;
        } else {

                return $hash;
        }

}

function abbreviation($hostname, $secret) {

        if (function_exists('mcrypt_ecb')) {

                $encrypted = base64_encode( mcrypt_ecb(MCRYPT_BLOWFISH, $secret, $hostname, MCRYPT_ENCRYPT) );
                return urlencode ( $encrypted );

        } else {

                return $hostname;

        }

}

